@@ -47,6 +47,7 @@ class UnlockDialog(BaseWindow):
 
         self.real_name = None
         self.user_name = None
+        self.auth_info = None
 
         self.bounce_rect = None
         self.bounce_count = 0
@@ -68,7 +69,7 @@ class UnlockDialog(BaseWindow):
 
         self.box.pack_start(self.realname_label, False, False, 10)
 
-        self.entry_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
+        self.entry_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, halign=Gtk.Align.CENTER)
 
         self.box.pack_start(self.entry_box, True, True, 2)
 
@@ -120,6 +121,11 @@ class UnlockDialog(BaseWindow):
         self.auth_message_label.set_alignment(0.5, 0.5)
         vbox_messages.pack_start(self.auth_message_label, False, False, 2)
 
+        self.authinfo_label = Gtk.Label(None)
+        self.authinfo_label.set_alignment(0, 0.5)
+        self.authinfo_label.set_halign(Gtk.Align.CENTER)
+        vbox_messages.pack_start(self.authinfo_label, False, False, 2)
+
         self.box.pack_start(vbox_messages, False, False, 0)
 
         self.real_name = utils.get_user_display_name()
@@ -128,12 +134,11 @@ class UnlockDialog(BaseWindow):
         self.update_realname_label()
 
         self.account_client = singletons.AccountsServiceClient
-        if self.account_client.is_loaded:
-            self.on_account_client_loaded(self.account_client)
-        else:
-            trackers.con_tracker_get().connect(self.account_client,
-                                               "account-loaded",
-                                               self.on_account_client_loaded)
+
+        self.set_user_details()
+        trackers.con_tracker_get().connect(self.account_client,
+                                           "accounts-ready",
+                                           self.on_accounts_ready)
 
         self.keymap = Gdk.Keymap.get_default()
 
@@ -160,19 +165,20 @@ class UnlockDialog(BaseWindow):
         trackers.con_tracker_get().connect(self.auth_client,
                                            "auth-prompt",
                                            self.on_authentication_prompt_changed)
+        trackers.con_tracker_get().connect(self.auth_client,
+                                           "auth-info",
+                                           self.on_authentication_info_changed)
+
 
         self.box.show_all()
+        self.password_entry.hide()
+        self.auth_unlock_button.hide()
 
     def initialize_auth_client(self):
         return self.auth_client.initialize()
 
-    def cancel_auth_client(self):
-        self.clear_entry()
-
-        self.auth_client.cancel()
-
     def on_authentication_success(self, auth_client):
-        self.clear_entry()
+        self.set_busy(False)
         self.emit("authenticate-success")
 
     def on_authentication_failure(self, auth_client):
@@ -180,9 +186,12 @@ class UnlockDialog(BaseWindow):
         Called upon authentication failure, clears the password, sets an error message,
         and refocuses the password entry.
         """
-        self.clear_entry()
+        self.set_busy(False)
         self.auth_message_label.set_text(_("Incorrect password"))
 
+        self.password_entry.hide()
+        self.auth_unlock_button.hide()
+
         self.emit("authenticate-failure")
         self.emit("uninhibit-timeout")
 
@@ -190,6 +199,9 @@ class UnlockDialog(BaseWindow):
         self.emit("authenticate-cancel")
 
     def on_authentication_busy_changed(self, auth_client, busy):
+        self.set_busy(busy)
+
+    def set_busy(self, busy):
         if busy:
             self.auth_message_label.set_text("")
             self.clear_entry()
@@ -200,8 +212,13 @@ class UnlockDialog(BaseWindow):
             self.entry_box.set_sensitive(True)
             self.password_entry.stop_progress()
             self.password_entry.set_placeholder_text (self.password_entry.placeholder_text)
+            self.auth_info = ""
+            self.update_authinfo_label()
 
     def on_authentication_prompt_changed(self, auth_client, prompt):
+        self.password_entry.show_all()
+        self.auth_unlock_button.show_all()
+
         if "password:" in prompt.lower():
             prompt = _("Please enter your password...")
         else:
@@ -210,11 +227,16 @@ class UnlockDialog(BaseWindow):
         self.password_entry.placeholder_text = prompt
         self.password_entry.set_placeholder_text(self.password_entry.placeholder_text)
 
+    def on_authentication_info_changed(self, auth_client, info):
+        self.auth_info = info
+        self.update_authinfo_label()
+
     def cancel(self):
         """
         Clears the auth message text if we have any.
         """
-        self.auth_message_label.set_text("")
+        self.auth_client.cancel()
+        self.clear_entry()
 
     def queue_key_event(self, event):
         """
@@ -236,16 +258,23 @@ class UnlockDialog(BaseWindow):
         else:
             self.capslock_label.set_text("")
 
-    def on_account_client_loaded(self, client):
+    def on_accounts_ready(self, client):
         """
         Handler for the AccountsService - requests the user real name and .face image.
         """
-        if client.get_real_name() != None:
-            self.real_name = client.get_real_name()
+        trackers.con_tracker_get().disconnect(self.account_client,
+                                              "accounts-ready",
+                                              self.on_accounts_ready)
+
+        self.set_user_details()
+
+    def set_user_details(self):
+        if self.account_client.get_real_name() is not None:
+            self.real_name = self.account_client.get_real_name()
             self.update_realname_label()
 
-        if client.get_face_path() != None:
-            self.face_image.set_from_path(client.get_face_path())
+        if self.account_client.get_face_path() is not None:
+            self.face_image.set_from_path(self.account_client.get_face_path())
             self.face_image.show()
 
     def on_password_entry_text_changed(self, editable):
@@ -302,6 +331,8 @@ class UnlockDialog(BaseWindow):
         Clear the password entry widget.
         """
         self.password_entry.set_text("")
+        self.auth_info = ""
+        self.update_authinfo_label()
 
     def update_realname_label(self):
         """
@@ -309,13 +340,19 @@ class UnlockDialog(BaseWindow):
         """
         self.realname_label.set_text(self.real_name)
 
+    def update_authinfo_label(self):
+        """
+        Updates the auth info label to the current auth_info message.
+        """
+        self.authinfo_label.set_text(self.auth_info)
+
     def blink(self):
         GObject.timeout_add(75, self.on_blink_tick)
 
     def on_blink_tick(self, data=None):
         window = self.get_window()
 
-        if window == None:
+        if window is None:
             return False
 
         x, y = window.get_position()
